@@ -53,7 +53,8 @@ typedef enum {
   BACKGROUND_CENTERED,
   BACKGROUND_SCALED,
   BACKGROUND_STRETCHED,
-  BACKGROUND_TILED
+  BACKGROUND_TILED,
+  BACKGROUND_CROPPED
 } BackgroundMode;
 
 GType background_mode_get_type (void) G_GNUC_CONST;
@@ -92,20 +93,15 @@ struct _BackgroundManagerClass
 
 GType                 background_manager_get_type          (void) G_GNUC_CONST;
 
-gboolean              background_manager_set_background 
+gboolean              background_manager_set_background
                             (BackgroundManager   *manager,
                              gint                 window_xid,
                              const gchar         *filename,
-                             const gchar         *top_bar,
-                             const gchar         *left_bar,
+                             const gchar         *cache,
                              guint16              red,
                              guint16              green,
                              guint16              blue,
                              BackgroundMode       mode,
-                             gint32               top_offset,
-                             gint32               bottom_offset,
-                             gint32               left_offset,
-                             gint32               right_offset,
                              gint                *pixmap_xid,
                              GError             **error);
 
