@@ -643,7 +643,7 @@ static void read_menu_conf(const char *filename, GtkTreeStore *menu_tree,
 					key,
                                         TREE_MODEL_LOCALIZED_NAME,
                                         g_strdup (dgettext (GETTEXT_PACKAGE,
-                                                            key)),
+                                                            (gchar *)key)),
 					TREE_MODEL_ICON,
 					folder_icon,
 					TREE_MODEL_THUMB_ICON,
