@@ -62,9 +62,10 @@
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcelllayout.h>
 
+#include <libhildondesktop/hildon-desktop-background.h>
+
 #include "hd-home-background-dialog.h"
 #include "hd-home-l10n.h"
-#include "hd-home-background.h"
 #include "background-manager/hildon-background-manager.h"
 
 #define BG_DESKTOP_GROUP           "Desktop Entry"
@@ -93,12 +94,13 @@
 
 G_DEFINE_TYPE (HDHomeBackgroundDialog, hd_home_background_dialog, GTK_TYPE_DIALOG);
 
-enum 
-{ 	 
-  BG_IMAGE_NAME, 	 
-  BG_IMAGE_FILENAME, 	 
+enum
+{
+  BG_IMAGE_NAME,
+  BG_IMAGE_FILENAME,
+  BG_IMAGE_CACHE,
   BG_IMAGE_PRIORITY
-}; 
+};
 
 enum
 {
@@ -110,15 +112,15 @@ enum
 
 struct _HDHomeBackgroundDialogPrivate
 {
-  gchar            *background_dir;
-  GtkListStore     *combobox_contents;
-  GtkTreePath      *custom_image;
-  GtkWidget        *color_button;
-  GtkWidget        *img_combo;
-  GtkWidget        *mode_combo;
-  gpointer          osso_context;
-  
-  HDHomeBackground *background;
+  gchar                        *background_dir;
+  GtkListStore                 *combobox_contents;
+  GtkTreePath                  *custom_image;
+  GtkWidget                    *color_button;
+  GtkWidget                    *img_combo;
+  GtkWidget                    *mode_combo;
+  gpointer                      osso_context;
+
+  HildonDesktopBackground      *background;
 };
 
 static void
@@ -135,7 +137,7 @@ hd_home_background_dialog_get_property (GObject      *gobject,
 
 
 static void
-hd_home_background_dialog_response (GtkDialog *dialog, 
+hd_home_background_dialog_response (GtkDialog *dialog,
                                     gint       arg);
 static void
 hd_home_background_dialog_set_background_dir (HDHomeBackgroundDialog *dialog,
@@ -146,8 +148,8 @@ hd_home_background_dialog_background_dir_changed
                                     (HDHomeBackgroundDialog *dialog);
 
 static void
-hd_home_background_dialog_set_background (HDHomeBackgroundDialog *dialog,
-                                          HDHomeBackground *background);
+hd_home_background_dialog_set_background (HDHomeBackgroundDialog   *dialog,
+                                          HildonDesktopBackground  *background);
 static void
 hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog);
 
@@ -171,14 +173,11 @@ imagename_from_filename (const gchar *filename)
   gchar *last_dot, *c;
 
   tmp = g_filename_from_uri (filename, NULL, NULL);
-  g_debug ("Got filename %s", tmp);
   if (!tmp)
     tmp = g_strdup (filename);
 
   imagename = g_filename_display_basename (tmp);
 
-  g_debug ("Got imagename %s", imagename);
-
   c = imagename;
   last_dot = NULL;
 
@@ -244,7 +243,7 @@ hd_home_background_dialog_class_init (HDHomeBackgroundDialogClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_BACKGROUND_DIR,
                                    pspec);
-  
+
   pspec = g_param_spec_pointer ("osso-context",
                                 "OSSO Context",
                                 "OSSO Context",
@@ -253,11 +252,11 @@ hd_home_background_dialog_class_init (HDHomeBackgroundDialogClass *klass)
   g_object_class_install_property (object_class,
                                    PROP_OSSO_CONTEXT,
                                    pspec);
-  
+
   pspec = g_param_spec_object ("background",
                                "Background",
                                "Current background",
-                               HD_TYPE_HOME_BACKGROUND,
+                               HILDON_DESKTOP_TYPE_BACKGROUND,
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
   g_object_class_install_property (object_class,
@@ -284,7 +283,8 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
     HH_SET_BG_MODE_CENTERED,
     HH_SET_BG_MODE_SCALED,
     HH_SET_BG_MODE_STRETCHED,
-    HH_SET_BG_MODE_TILED
+    HH_SET_BG_MODE_TILED,
+    HH_SET_BG_MODE_CROPPED
   };
 
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
@@ -302,18 +302,18 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
 
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_window_set_title (GTK_WINDOW (dialog), HH_SET_BG_TITLE);
-  
+
   /* Hildon Caption HBoxes */
   hbox_color = gtk_hbox_new (FALSE, 10);
   hbox_image = gtk_hbox_new (FALSE, 10);
   hbox_mode  = gtk_hbox_new (FALSE, 10);
-  
+
   priv->color_button = hildon_color_button_new ();
-  
+
   g_signal_connect_swapped (priv->color_button, "notify::color",
                             G_CALLBACK (hd_home_background_dialog_color_changed),
                             dialog);
-  
+
   priv->img_combo = gtk_combo_box_new ();
 
   g_signal_connect_swapped (priv->img_combo, "changed",
@@ -322,24 +322,26 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
 
   renderer = gtk_cell_renderer_text_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->img_combo),
-                              renderer, 
+                              renderer,
                               TRUE);
 
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (priv->img_combo),
-                                  renderer, "text", 
+                                  renderer, "text",
                                   BG_IMAGE_NAME, NULL);
-  
+
   priv->mode_combo = gtk_combo_box_new_text ();
-  
-  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo), 
+
+  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo),
                              image_modes[BACKGROUND_CENTERED]);
-  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo), 
+  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo),
                              image_modes[BACKGROUND_SCALED]);
-  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo), 
+  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo),
                              image_modes[BACKGROUND_STRETCHED]);
-  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo), 
+  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo),
                              image_modes[BACKGROUND_TILED]);
-  
+  gtk_combo_box_append_text (GTK_COMBO_BOX (priv->mode_combo),
+                             image_modes[BACKGROUND_CROPPED]);
+
   g_signal_connect_swapped (priv->mode_combo, "changed",
                             G_CALLBACK (hd_home_background_dialog_mode_changed),
                             dialog);
@@ -364,9 +366,9 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
                           priv->mode_combo,
                           NULL,
                           HILDON_CAPTION_OPTIONAL);
-     
+
   gtk_box_pack_start (GTK_BOX (hbox_color), color_caption,
-                      FALSE, FALSE, 0); 
+                      FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox_image), image_caption,
                       TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox_mode), mode_caption,
@@ -381,7 +383,7 @@ hd_home_background_dialog_init (HDHomeBackgroundDialog *dialog)
 
   hildon_caption_set_child_expand (HILDON_CAPTION (image_caption), TRUE);
   hildon_caption_set_child_expand (HILDON_CAPTION (mode_caption), TRUE);
-  
+
   /* Let the WM take care of the positioning */
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_NONE);
 
@@ -417,7 +419,7 @@ hd_home_background_dialog_set_property (GObject      *gobject,
                                           priv->osso_context);
 #endif
         break;
-        
+
     case PROP_BACKGROUND:
         hd_home_background_dialog_set_background (
                            HD_HOME_BACKGROUND_DIALOG (gobject),
@@ -511,18 +513,18 @@ hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
   fdialog = gtk_file_chooser_dialog_new (HILDON_HOME_FILE_CHOOSER_TITLE,
 					 GTK_WINDOW (dialog),
 					 GTK_FILE_CHOOSER_ACTION_OPEN,
-                                         HILDON_HOME_FILE_CHOOSER_SELECT, GTK_RESPONSE_OK, 
-                                         HILDON_HOME_FILE_CHOOSER_EMPTY, GTK_RESPONSE_CANCEL, 
+                                         HILDON_HOME_FILE_CHOOSER_SELECT, GTK_RESPONSE_OK,
+                                         HILDON_HOME_FILE_CHOOSER_EMPTY, GTK_RESPONSE_CANCEL,
 					 NULL);
 #endif
-  
+
 #ifdef HAVE_LIBHILDONHELP
   if (priv->osso_context)
-    hildon_help_dialog_help_enable (GTK_DIALOG(fdialog), 
+    hildon_help_dialog_help_enable (GTK_DIALOG(fdialog),
                                     HH_HELP_SELECT_IMAGE,
                                     priv->osso_context);
 #endif
-        
+
   mime_type_filter = gtk_file_filter_new();
   gtk_file_filter_add_mime_type (mime_type_filter, "image/jpeg");
   gtk_file_filter_add_mime_type (mime_type_filter, "image/gif");
@@ -540,23 +542,23 @@ hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
 
   if (!gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (fdialog),
                                             image_dir))
-    g_warning ("Couldn't set default image dir for dialog %s", 
+    g_warning ("Couldn't set default image dir for dialog %s",
                image_dir);
 
   g_free (image_dir);
-  
+
   response = gtk_dialog_run (GTK_DIALOG (fdialog));
-    
-  if (response == GTK_RESPONSE_OK) 
+
+  if (response == GTK_RESPONSE_OK)
     {
       const gchar  *filename;
 
       filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fdialog));
 
-      if (filename) 
+      if (filename)
         {
           GtkTreeModel *model;
-          GtkTreeIter   iter;            
+          GtkTreeIter   iter;
           gchar        *name;
 
           name = imagename_from_filename (filename);
@@ -575,7 +577,7 @@ hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
 
           gtk_list_store_append (priv->combobox_contents,
                                  &iter);
-          
+
           priv->custom_image = gtk_tree_model_get_path (model,
                                                         &iter);
 
@@ -583,6 +585,7 @@ hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
                               &iter,
                               BG_IMAGE_NAME, name?name:filename,
                               BG_IMAGE_FILENAME, filename,
+                              BG_IMAGE_CACHE, "",
                               BG_IMAGE_PRIORITY, G_MAXINT,
                               -1);
 
@@ -591,7 +594,7 @@ hd_home_background_dialog_file_select (HDHomeBackgroundDialog *dialog)
           g_free (name);
         }
 
-    }  
+    }
 
   gtk_widget_destroy (fdialog);
 
@@ -614,8 +617,9 @@ hd_home_background_dialog_filename_changed (HDHomeBackgroundDialog *dialog)
 {
   HDHomeBackgroundDialogPrivate    *priv;
   GtkTreeIter                       iter;
-  gchar                            *filename;
-  
+  gchar                            *filename = NULL;
+  gchar                            *cache = NULL;
+
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
   gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->img_combo),
@@ -624,8 +628,12 @@ hd_home_background_dialog_filename_changed (HDHomeBackgroundDialog *dialog)
   gtk_tree_model_get (GTK_TREE_MODEL (priv->combobox_contents),
                       &iter,
                       BG_IMAGE_FILENAME, &filename,
+                      BG_IMAGE_CACHE, &cache,
                       -1);
 
+  if (!priv->background)
+    goto out;
+
   if (filename)
     g_object_set (G_OBJECT (priv->background),
                   "filename", filename,
@@ -635,7 +643,17 @@ hd_home_background_dialog_filename_changed (HDHomeBackgroundDialog *dialog)
                   "filename", HD_HOME_BACKGROUND_NO_IMAGE,
                   NULL);
 
+  if (cache)
+    g_object_set (G_OBJECT (priv->background),
+                  "cache", cache,
+                  NULL);
+  else
+    g_object_set (G_OBJECT (priv->background),
+                  "cache", "",
+                  NULL);
+out:
   g_free (filename);
+  g_free (cache);
 }
 
 static void
@@ -683,15 +701,13 @@ hd_home_background_dialog_mode_changed (HDHomeBackgroundDialog *dialog)
 
   mode = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->mode_combo));
 
-  g_debug ("Setting mode on dialog background to %i", mode);
-
   g_object_set (G_OBJECT (priv->background),
                 "mode", mode,
                 NULL);
 }
 
 static void
-hd_home_background_dialog_background_dir_changed 
+hd_home_background_dialog_background_dir_changed
                                     (HDHomeBackgroundDialog *dialog)
 {
   HDHomeBackgroundDialogPrivate *priv;
@@ -709,10 +725,12 @@ hd_home_background_dialog_background_dir_changed
   else
     {
       priv->combobox_contents =
-          gtk_list_store_new (3,
+          gtk_list_store_new (4,
                               G_TYPE_STRING, /*localised descriptive name */
                               G_TYPE_STRING, /* image file path & name */
-                              G_TYPE_INT     /* image priority */);
+                              G_TYPE_STRING, /* image local cache */
+                              G_TYPE_INT);   /* image priority */
+
       gtk_combo_box_set_model (GTK_COMBO_BOX (priv->img_combo),
                                GTK_TREE_MODEL (priv->combobox_contents));
     }
@@ -722,8 +740,10 @@ hd_home_background_dialog_background_dir_changed
   gtk_list_store_set (priv->combobox_contents,
                       &iterator,
                       BG_IMAGE_NAME, HH_SET_BG_IMAGE_NONE,
-                      BG_IMAGE_FILENAME, NULL,
-                      BG_IMAGE_PRIORITY, 0, -1);
+                      BG_IMAGE_FILENAME, "",
+                      BG_IMAGE_CACHE, "",
+                      BG_IMAGE_PRIORITY, 0,
+                      -1);
 
   /* It's the default */
   gtk_combo_box_set_active_iter (GTK_COMBO_BOX (priv->img_combo),
@@ -800,6 +820,7 @@ hd_home_background_dialog_background_dir_changed
                            * empty strings */
                           ((image_name && *image_name) ? _(image_name) : image_path),
                           BG_IMAGE_FILENAME, image_path,
+                          BG_IMAGE_CACHE, "",
                           BG_IMAGE_PRIORITY, image_order,
                           -1);
 
@@ -812,15 +833,15 @@ end_of_loop:
       g_free (filename);
       g_free (image_name);
       g_free (image_path);
-    } 
+    }
+
+  if (dir)
+    g_dir_close (dir);
 
-  if (dir) 
-    g_dir_close (dir);             
-      
   g_key_file_free (kfile);
 
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->combobox_contents),
-                                        BG_IMAGE_PRIORITY, 
+                                        BG_IMAGE_PRIORITY,
                                         GTK_SORT_ASCENDING);
 }
 
@@ -829,6 +850,7 @@ hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog)
 {
   HDHomeBackgroundDialogPrivate    *priv;
   gchar                            *filename;
+  gchar                            *cache;
   GdkColor                         *color;
   BackgroundMode                    mode;
 
@@ -837,14 +859,15 @@ hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog)
 
   if (!priv->background)
     return;
-  
+
   g_object_get (G_OBJECT (priv->background),
                 "color", &color,
                 "filename", &filename,
+                "cache", &cache,
                 "mode", &mode,
                 NULL);
 
-  if (filename && *filename && 
+  if (filename && *filename &&
       !g_str_equal (filename, HD_HOME_BACKGROUND_NO_IMAGE))
     {
       gboolean      valid;
@@ -868,6 +891,8 @@ hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog)
             break;
 
           valid = gtk_tree_model_iter_next (model, &iter);
+
+          g_free (existing_filename);
         }
 
       if (!valid)
@@ -879,6 +904,7 @@ hd_home_background_dialog_sync_from_background (HDHomeBackgroundDialog *dialog)
           gtk_list_store_set (priv->combobox_contents,
                               &iter,
                               BG_IMAGE_FILENAME, filename,
+                              BG_IMAGE_CACHE, cache,
                               BG_IMAGE_NAME, image_name,
                               BG_IMAGE_PRIORITY, G_MAXINT,
                               -1);
@@ -913,7 +939,7 @@ hd_home_background_dialog_set_background_dir (HDHomeBackgroundDialog *dialog,
   g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog) && dir);
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
-  if (!priv->background_dir 
+  if (!priv->background_dir
       || !g_str_equal (dir, priv->background_dir))
   {
     g_free (priv->background_dir);
@@ -925,27 +951,27 @@ hd_home_background_dialog_set_background_dir (HDHomeBackgroundDialog *dialog,
 }
 
 static void
-hd_home_background_dialog_set_background (HDHomeBackgroundDialog *dialog,
-                                          HDHomeBackground *background)
+hd_home_background_dialog_set_background (HDHomeBackgroundDialog  *dialog,
+                                          HildonDesktopBackground *background)
 {
   HDHomeBackgroundDialogPrivate *priv;
-  
+
   g_return_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog) &&
-                    HD_IS_HOME_BACKGROUND (background));
+                    HILDON_DESKTOP_IS_BACKGROUND (background));
 
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
   if (priv->background)
     g_object_unref (priv->background);
 
-  priv->background = hd_home_background_copy (background);
+  priv->background = hildon_desktop_background_copy (background);
 
   g_object_notify (G_OBJECT (dialog), "background");
   hd_home_background_dialog_sync_from_background (dialog);
 
 }
 
-HDHomeBackground *
+HildonDesktopBackground *
 hd_home_background_dialog_get_background (HDHomeBackgroundDialog *dialog)
 {
   HDHomeBackgroundDialogPrivate *priv;
@@ -953,5 +979,5 @@ hd_home_background_dialog_get_background (HDHomeBackgroundDialog *dialog)
   g_return_val_if_fail (HD_IS_HOME_BACKGROUND_DIALOG (dialog), NULL);
   priv = HD_HOME_BACKGROUND_DIALOG_GET_PRIVATE (dialog);
 
-  return g_object_ref (priv->background);
+  return hildon_desktop_background_copy (priv->background);
 }
